#ifndef sithwm_h_included
#define sithwm_h_included

#include <X11/X.h>
#include <X11/Xatom.h>
#include <X11/Xlib.h>
#include <X11/Xmd.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <stdio.h>
#ifndef __cplusplus
#define _GNU_SOURCE
#endif
#include <string.h>

#ifndef NOLOG
# define LOG_INFO(...) printf(__VA_ARGS__)
# define LOG_ERROR(...) fprintf(stderr, __VA_ARGS__)
#else
# define LOG_INFO(...)
# define LOG_ERROR(...)
#endif

struct masktab;

#ifdef DEBUG
# define LOG_DEBUG(...) fprintf(stderr, __VA_ARGS__)
void prt_mask(unsigned int mask, const struct masktab*t);
#else
# define LOG_DEBUG(...)
#define prt_mask(m,t) ((void)0)
#endif

#define STRLEN (110)
extern XGCValues glob_gcv;
extern char global_buffer[STRLEN];

extern unsigned int     numlockmask;

/* Coordinates */

struct pos {
   int x,y;
};

struct area {
   struct pos pos;
   int w,h;
};

/* screen structure */

#define COL_BASE	('b')
#define COL_NO_OF	('j'-COL_BASE)

enum {INFOW_OFF, INFOW_CAPTION, INFOW_MAP, INFOW_MENU};

#define VIEW_HISTORY (2)
typedef struct ScreenInfo ScreenInfo;
struct ScreenInfo {
   Window infow;
   unsigned char infow_func;
   unsigned char do_infow_func;
   Window hilit;
   Window root;
   GC gc[COL_NO_OF];
   unsigned long pixel[COL_NO_OF];
   struct area area;
   struct pos hist[VIEW_HISTORY];
   unsigned int hist_index;
   unsigned int hilit_funcmask;
};

extern ScreenInfo   *screens;
extern ScreenInfo*current_screen;
extern int          num_screens;
extern char default_name[2];

/* client structure */

struct Client_d {
   Window  window;
   struct area area;
   char named;
   char sticky;
   char border_color;
   char remove;  /* set when client needs to be removed */
};

#define HISTORY (4)
typedef struct Client Client;
struct Client {
   struct Client_d dyn;
   char *name;
   ScreenInfo      *screen;
   Client  *next;
   Drawable icon;
   unsigned int icon_width, icon_height;
   Colormap        cmap;
   struct area hist[HISTORY];
   unsigned hist_index;
   struct pos last_cursor;
   struct pos last_screen;
};

typedef struct Application Application;
struct Application {
   char *res_name;
   char *res_class;
   int geometry_mask;
   struct area area;
   char sticky;
   Application *next;
};
extern Application *head_app;

/* Declarations for global variables in main.c */
extern const char   *colnams[COL_NO_OF];
extern const char   *opt_font;

/* Commonly used X information */
extern Display      *dpy;
extern XFontStruct  *font;
extern int	    font_height;

extern Atom xa_wm_state;
extern Atom xa_wm_delete;
extern Atom xa_wm_protos;
extern Atom xa_wm_delete;

/* Things that affect user interaction */
extern int              opt_bw;

extern struct area	opt_popup_area;
extern int		opt_popup_stack_width;
extern int		opt_popup_time_in;
extern int stacker_offset;
extern unsigned int infow_scaler;

#ifdef SNAP
extern int              opt_snap;
#endif

extern int solid_drag;

extern const char*lock_command;

/* Client tracking information */
extern Client *head_client;

/* client.c */
void make_new_client(Window w, ScreenInfo*s, int flags);
Client *find_client(Window w);
void place_client(Client *c);
void get_client_props(Client*client);

enum {

	/* Functions, normally shifted left */
	FUNC_NONE, FUNC_SPAWN, FUNC_MENU, FUNC_MOVE,
	FUNC_NEXT, FUNC_PAN, FUNC_CIRCUL, FUNC_HIST,
	FUNC_KILL, FUNC_LOWER, FUNC_RESIZE, FUNC_DRAGRESIZE,
	FUNC_NEXT_SCREEN,

	XACT_ENTER, XACT_NEW, XACT_DELETED,

	FUNC_MAX,

        /* Modifiers for the functions. */
	FMOD_MOVE_RESIZE= 0x1,
	FMOD_SET_VIEW	= 0x2,
	FMOD_RAISE	= 0x4,
	FMOD_SETMOUSE	= 0x8,
	FMOD_FOCUS	= 0x10,
	FMOD_MAXIMISE	= 0x20,
	FMOD_FIX	= 0x40,
	FMOD_MAP	= 0x80,
	FMOD_ABS	= 0x100,
	FMOD_INFO	= 0x200,
	FMOD_VIEW	= 0x400,
	FMOD_POPUP	= 0x800,
};

void select_client(Client *c, unsigned int func);
Client* select_window(Window w, unsigned int func);

#ifdef WINDOW_LOG
void log_window_event(const char*cmd, Client*c);
#else
#define log_window_event(x,y) ((void)1)
#endif

/* events.c */
struct Dragsweep{
   Window w;
   unsigned fl;
   struct pos pos;
};
extern struct Dragsweep dragsweep;

void event_main_loop(void);

/* misc.c */
unsigned int hash_val(void*p, size_t len, unsigned int s);
int handle_xerror(Display *dsply, XErrorEvent *e);
void spawn(const char*command);
int intersect(struct area *r1, struct area *r2);


/* screen.c */
void setview(void);

void toviewpos(Client *c, struct pos*p);
void toworldpos(Client *c, int x, int y, struct pos*p);
const char *map_state_string(int map_state);

extern unsigned int last_current_info;
void update_info_window(int pri);


void read_rcs(void);   
void do_menu(XEvent*e);

#ifdef __GNUC__
extern char*strdup(const char*);
#endif
extern int putenv(char*);


// sithfunc
struct masktab {
   const char *name;
   unsigned int mask;
};
extern struct masktab modifiers[];
extern const struct masktab funcmasks[];
void init_funcs(FILE*f);
void do_func(XEvent*e, KeySym key);
void func_grab(Window w, int butt_only);
unsigned int parse_modifiers(char *s);
unsigned int parse_mask(char *s, const struct masktab*t);
void parse_rc_line();

#define FUNC_SHIFT (24)

extern unsigned int setmodifiers[];


#endif
